#
# booleansPage.py - GUI for Booleans page in system-config-securitylevel
#
# Brent Fox <bfox@redhat.com>
# Dan Walsh <dwalsh@redhat.com>
#
# Copyright 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import string
import gtk
import gtk.glade
import os
import libxml2
import gobject
import sys
import tempfile

INSTALLPATH='/usr/share/system-config-selinux'
sys.path.append(INSTALLPATH)

import commands
ENFORCING=0
PERMISSIVE=1
DISABLED=2

##
## I18N
## 
PROGNAME="system-config-selinux"

import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME,
                    localedir="/usr/share/locale",
                    unicode=False,
                    codeset = 'utf-8')
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

from glob import fnmatch

class Translation:
    def __init__(self):
        self.translation={}
        fd=open(INSTALLPATH + "/selinux.tbl","r")
        lines=fd.readlines()
        fd.close()
        for i in lines:
            try:
                line=i.strip().split("_(\"")
                key=line[0].strip()
                category=line[1].split("\"")[0]
                value=line[2].split("\"")[0]
                self.translation[key]=(category,value)
            except:
                continue

    def match(self,key, filter=""):
        try:
            f=filter.lower()
            val=self.get_value(key).lower()
            k=key.lower()
            return val.find(f) >= 0 or k.find(f) >= 0
        except:
            return False

    def get_category(self,key):
        try:
            return _(self.translation[key][0])
        except:
            return _("Other")

    def get_value(self,key):
        try:
            return _(self.translation[key][1])
        except:
            return key

class Modifier:
    def __init__(self,name, on, save):
        self.on=on
        self.name=name
        self.save=save

    def set(self,value):
        self.on=value
        self.save=True

    def isOn(self):
        return self.on

class Boolean(Modifier):
    def __init__(self,name, val, save=False):
        Modifier.__init__(self,name, val, save)

class Modifiers:
    def __init__(self,store):
        self.modifiers={}
        self.translation=Translation()
        self.store=store
        self.store.clear()

    def add(self,name,val):
        if name == "targeted_policy":
            return
        category=self.translation.get_category(name)
        if not self.modifiers.has_key(category):
            self.modifiers[category]={}
            iter=self.store.append(None)
            self.modifiers[category]["iter"] = iter
            self.store.set_value(iter, 1, category)
            self.store.set_value(iter, 3, False)

        self.modifiers[category][name]=val;
        iter=self.store.append(self.modifiers[category]["iter"])
        self.store.set_value(iter, 0, val.isOn())
        self.store.set_value(iter, 1, self.translation.get_value(name))
        self.store.set_value(iter, 2, name)
        self.store.set_value(iter, 3, True)

    def set(self,name,val):
        category=self.translation.get_category(name)
        self.modifiers[category][name].set(val)

    def isBoolean(self,name):
        c=self.translation.get_category(name)
        return isinstance(self.modifiers[c][name], Boolean)

    def get_booleans(self):
        booleans={}
        for c in self.modifiers.keys():
            for n in self.modifiers[c].keys():
                if isinstance(self.modifiers[c][n], Boolean):
                    booleans[n]=self.modifiers[c][n]
        return booleans

class booleansPage:
    def __init__(self, xml, doDebug=None):
        self.xml = xml
        self.types=[]
        self.selinuxsupport = True
        self.translation = Translation()
        self.typechanged = False
        self.doDebug = doDebug

        # Bring in widgets from glade file.
        self.typeHBox = xml.get_widget("typeHBox")
        self.booleanSW = xml.get_widget("booleanSW")
        self.booleansFilter = xml.get_widget("booleansFilter")
        self.booleansFilter.connect("focus_out_event", self.filter_changed)
        self.booleansFilter.connect("activate", self.filter_changed)
        
        self.booleansView = xml.get_widget("booleansView")
        self.typeLabel = xml.get_widget("typeLabel")
        self.modifySeparator = xml.get_widget("modifySeparator")

        listStore = gtk.ListStore(gobject.TYPE_STRING)
        cell = gtk.CellRendererText()

        self.booleansStore = gtk.TreeStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN)
        self.booleansStore.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        self.booleansView.set_model(self.booleansStore)

        checkbox = gtk.CellRendererToggle()
        checkbox.connect("toggled", self.boolean_toggled)
        col = gtk.TreeViewColumn('', checkbox, active = 0,visible=3)
        col.set_fixed_width(20)
        col.set_clickable(True)
        self.booleansView.append_column(col)

        col = gtk.TreeViewColumn("", gtk.CellRendererText(), text=1)
        self.booleansView.append_column(col)
        self.filter=""
        self.refreshBooleans(self.filter)
            
    def filter_changed(self, *arg):
        filter =  arg[0].get_text()
        if filter != self.filter:
            self.refreshBooleans(filter)
            self.filter=filter
        
    def use_menus(self):
        return False
    
    def get_description(self):
        return _("Boolean")

    def refreshBooleans(self, filter=None):
        self.modifiers=Modifiers(self.booleansStore)
        booleansList=commands.getoutput("/usr/sbin/getsebool -a").split("\n")
        for i in booleansList:
            rec=i.split()
            name=rec[0]
            if self.translation.match(name, filter):
                if rec[2]=="on" or rec[2]=="active":
                    on=1
                else:
                    on=0
                self.modifiers.add(name,Boolean(name,on))

    def boolean_toggled(self, widget, row):
        if len(row) == 1:
            return
        iter = self.booleansStore.get_iter(row)
        val = self.booleansStore.get_value(iter, 0)
        key = self.booleansStore.get_value(iter, 2)
        self.booleansStore.set_value(iter, 0 , not val)
        self.modifiers.set(key, not val)

        setsebool="/usr/sbin/setsebool -P %s=%d" % (key, not val)
        commands.getstatusoutput(setsebool)
