#! /usr/bin/python
# Copyright (C) 2007 Red Hat 
# see file 'COPYING' for use and warranty information
#
# policygentool is a tool for the initial generation of SELinux policy
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License as
#    published by the Free Software Foundation; either version 2 of
#    the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA     
#                                        02111-1307  USA
#
#  
import os, sys, getopt, stat
import re
from templates import executable
from templates import var_spool
from templates import var_lib
from templates import var_log
from templates import var_run
from templates import tmp
from templates import rw
from templates import network
from templates import script
import seobject

##
## I18N
## 
PROGNAME="system-config-selinux"

import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME,
                    localedir="/usr/share/locale",
                    unicode=False,
                    codeset = 'utf-8')
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode


class policy:
	Daemon = 0
	INETD = 1
	USER = 2
	CGI = 3
	def __init__(self, name, program, type):
                ports = seobject.portRecords()
                self.dict = ports.get_all()

		self.DEFAULT_DIRS = {}
		self.DEFAULT_DIRS["rw"] = ["rw", [], rw];
		self.DEFAULT_DIRS["tmp"] = ["tmp", [], tmp];
		self.DEFAULT_DIRS["/var/spool"] = ["var_spool", [], var_spool];
		self.DEFAULT_DIRS["/var/lib"] = ["var_lib", [], var_lib];
		self.DEFAULT_DIRS["/var/log"] = ["var_log", [], var_log];
		self.DEFAULT_DIRS["/var/run"] = ["var_run", [], var_run];

		self.DEFAULT_TYPES = (( self.generate_daemon_types, self.generate_daemon_rules), ( self.generate_inetd_types, self.generate_inetd_rules), ( self.generate_userapp_types, self.generate_userapp_rules), ( self.generate_cgi_types, self.generate_cgi_rules))
		if name == "":
			raise ValueError(_("You must enter a name for your confined process"))
		if type == self.CGI:
			self.name = "httpd_%s_script" % name
		else:
			self.name = name
		self.file_name = name
		if program == "":
			raise ValueError(_("You must enter the executable path for your confined process"))
		self.program = program
		self.type = type
		self.in_tcp = []
		self.in_udp = []
		self.out_tcp = []
		self.out_udp = []
		self.use_tmp = False
		self.use_uid = False
		self.use_pam = False
		self.use_syslog = False
		self.files = {}
		self.dirs = {}
                self.found_tcp_ports=[]
                self.found_udp_ports=[]
                self.need_tcp_type=False
                self.need_udp_type=False
		
        def find_port(self, port):
            for begin,end in self.dict.keys():
                if port >= begin and port <= end:
                    return self.dict[begin,end]
            return  None

	def __verify_ports(self, ports):
		if ports == "":
			return []
		max_port=2**16
		try:
			temp = []
			for p in ports.split(","):
				i = int(p.strip())
				if i < 1 or i > max_port:
					raise  ValueError()
				temp.append(i)
			return temp
		except ValueError:
			raise  ValueError(_("Ports must be be numbers from 1 to %d " % max_port ))

	def set_in_tcp(self, ports):
		self.in_tcp = self.__verify_ports(ports)

	def set_in_udp(self, ports):
		self.in_udp = self.__verify_ports(ports)

	def set_out_tcp(self, ports):
		self.out_tcp = self.__verify_ports(ports)

	def set_out_udp(self, ports):
		self.out_udp = self.__verify_ports(ports)

	def set_use_syslog(self, val):
		if val != True and val != False:
			raise  ValueError(_("use_syslog must be a boolean value "))
            
		self.use_syslog = val
		
	def set_use_pam(self, val):
		if val != True and val != False:
			raise  ValueError(_("use_pam must be a boolean value "))

		self.use_pam = val
		
	def set_use_tmp(self, val):
            if val == True:
		self.DEFAULT_DIRS["tmp"][1].append("/tmp");
                return
            if val == False:
		self.DEFAULT_DIRS["tmp"][1]=[]
                return
            raise  ValueError(_("use_tmp must be a boolean value "))
            
		
	def set_use_uid(self, val):
		if val != True and val != False:
			raise  ValueError(_("use_uid must be a boolean value "))
            
		self.use_uid = val

	def generate_uid_rules(self):
		return re.sub("TEMPLATETYPE", self.name, executable.te_uid_rules)
	def generate_syslog_rules(self):
		return re.sub("TEMPLATETYPE", self.name, executable.te_syslog_rules)

	def generate_pam_rules(self):
		return re.sub("TEMPLATETYPE", self.name, executable.te_pam_rules)
            
	def generate_network_types(self):
                for i in self.in_tcp:
                    rec = self.find_port(int(i))
                    if rec == None:
                        self.need_tcp_type = True;
                    else:
                        port_name = rec[0][:-2]
                        line = "corenet_tcp_bind_%s(%s_t)\n" % (port_name, self.name)
                        if line not in self.found_tcp_ports:
                            self.found_tcp_ports.append(line)

                for i in self.out_tcp:
                    rec = self.find_port(int(i))
                    if rec == None:
                        self.need_tcp_type = True;
                    else:
                        port_name = rec[0][:-2]
                        line = "corenet_tcp_connect_%s(%s_t)\n" % (port_name, self.name)
                        if line not in self.found_tcp_ports:
                            self.found_tcp_ports.append(line)
                        
                for i in self.in_udp:
                    rec = self.find_port(int(i))
                    if rec == None:
                        self.need_udp_type = True;
                    else:
                        port_name = rec[0][:-2]
                        line = "corenet_udp_bind_%s(%s_t)\n" % (port_name, self.name)
                        if line not in self.found_udp_ports:
                            self.found_udp_ports.append(line)
                
                if self.need_udp_type == True or self.need_tcp_type == True:
                    return re.sub("TEMPLATETYPE", self.name, network.te_port_types)
		return ""
	
	def __find_path(self, file):
		for d in self.DEFAULT_DIRS:
			if file.find(d) == 0:
				self.DEFAULT_DIRS[d][1].append(file)
				return self.DEFAULT_DIRS[d]
		self.DEFAULT_DIRS["rw"][1].append(file)
		return self.DEFAULT_DIRS["rw"]
	
	def add_file(self, file):
		self.files[file] = self.__find_path(file)

	def add_dir(self, file):
		self.dirs[file] = self.__find_path(file)
		
	def generate_network_rules(self):
		newte = ""
		tcp = len(self.in_tcp) + len(self.out_tcp)
		udp = len(self.in_udp) + len(self.out_udp)
		if tcp >  0 or udp > 0:
                    newte = "\n"

                    newte += re.sub("TEMPLATETYPE", self.name, network.te_network)
                    
                    if tcp > 0:
                        newte += "\n"
                        newte += re.sub("TEMPLATETYPE", self.name, network.te_tcp)
                        if self.need_tcp_type and len(self.out_tcp) > 0:
                            newte += re.sub("TEMPLATETYPE", self.name, network.te_out_need_port_tcp)
                        if self.need_tcp_type and len(self.in_tcp) > 0:
                            newte += re.sub("TEMPLATETYPE", self.name, network.te_in_need_port_tcp)
                        if len(self.in_tcp) > 0:
                            newte += re.sub("TEMPLATETYPE", self.name, network.te_in_tcp)
                        for i in self.found_tcp_ports:
                            newte += i

                    if udp > 0:
                        newte += "\n"
                        newte += re.sub("TEMPLATETYPE", self.name, network.te_udp)
                        if self.need_udp_type and len(self.in_udp) > 0:
                            newte += re.sub("TEMPLATETYPE", self.name, network.te_in_need_port_udp)
                        if len(self.in_udp) > 0:
                            newte += re.sub("TEMPLATETYPE", self.name, network.te_in_udp)
                        for i in self.found_udp_ports:
                            newte += i
		return newte
	
	def generate_cgi_types(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.file_name, executable.te_cgi_types)
		return newte
	
	def generate_userapp_types(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_userapp_types)
		return newte
	
	def generate_inetd_types(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_inetd_types)
		return newte
	
	def generate_daemon_types(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_daemon_types)
		return newte
	
	def generate_cgi_te(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_cgi_types)
		return newte

	def generate_daemon_rules(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_daemon_rules)
		return newte
	
	def generate_userapp_rules(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_userapp_rules)
		return newte
	
	def generate_inetd_rules(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_inetd_rules)
		return newte
	
	def generate_cgi_rules(self):
		newte = ""
		newte += re.sub("TEMPLATETYPE", self.name, executable.te_cgi_rules)
		return newte
	
	def generate_if(self):
		newif = re.sub("TEMPLATETYPE", self.name, executable.if_rules)
		
		for d in self.DEFAULT_DIRS:
			if len(self.DEFAULT_DIRS[d][1]) > 0:
				newif += re.sub("TEMPLATETYPE", self.name, self.DEFAULT_DIRS[d][2].if_rules)
                                for i in self.DEFAULT_DIRS[d][1]:
                                        if os.path.exists(i) and stat.S_ISSOCK(os.stat(i)[stat.ST_MODE]):
                                            newif += re.sub("TEMPLATETYPE", self.name, self.DEFAULT_DIRS[d][2].if_stream_rules)
                                            break
		return newif

	def generate_default_types(self):
		return self.DEFAULT_TYPES[self.type][0]()
		
	def generate_default_rules(self):
		return self.DEFAULT_TYPES[self.type][1]()
		
	def generate_te(self):
		newte = self.generate_default_types()
		for d in self.DEFAULT_DIRS:
			if len(self.DEFAULT_DIRS[d][1]) > 0:
				# CGI scripts already have a rw_t 
				if self.type != self.CGI or d != "rw":
					newte += re.sub("TEMPLATETYPE", self.name, self.DEFAULT_DIRS[d][2].te_types)

		newte += self.generate_network_types()
		if self.use_tmp:
			newte += self.generate_tmp_types()

		newte += self.generate_default_rules()
		for d in self.DEFAULT_DIRS:
			if len(self.DEFAULT_DIRS[d][1]) > 0:
				newte += re.sub("TEMPLATETYPE", self.name, self.DEFAULT_DIRS[d][2].te_rules)
                                for i in self.DEFAULT_DIRS[d][1]:
                                        if os.path.exists(i) and stat.S_ISSOCK(os.stat(i)[stat.ST_MODE]):
                                            newte += re.sub("TEMPLATETYPE", self.name, self.DEFAULT_DIRS[d][2].te_stream_rules)
                                            break

		newte += self.generate_network_rules()
		if self.use_tmp:
			newte += self.generate_tmp_rules()
				
		if self.use_uid:
			newte += self.generate_uid_rules()		

		if self.use_syslog:
			newte += self.generate_syslog_rules()		

		if self.use_pam:
			newte += self.generate_pam_rules()		

		return newte
		
	def generate_fc(self):
		newfc = ""
		t1 = re.sub("EXECUTABLE", self.program, executable.fc_file)
		newfc += re.sub("TEMPLATETYPE", self.name, t1)

		for i in self.files.keys():
                        if os.path.exists(i) and stat.S_ISSOCK(os.stat(i)[stat.ST_MODE]):
                            t1 = re.sub("TEMPLATETYPE", self.name, self.files[i][2].fc_sock_file)
                        else:
                            t1 = re.sub("TEMPLATETYPE", self.name, self.files[i][2].fc_file)
			t2 = re.sub("FILENAME", i, t1)
			newfc += re.sub("FILETYPE", self.files[i][0], t2)

		for i in self.dirs.keys():
			t1 = re.sub("TEMPLATETYPE", self.name, self.dirs[i][2].fc_dir)
			t2 = re.sub("FILENAME", i, t1)
			newfc += re.sub("FILETYPE", self.dirs[i][0], t2)

		return newfc
	
	def generate_sh(self):
		newsh = re.sub("TEMPLATETYPE", self.name, script.compile)
		newsh = re.sub("PACKAGEFILENAME", self.file_name, newsh)
		newsh += re.sub("FILENAME", self.program, script.restorecon)

		for i in self.files.keys():
			newsh += re.sub("FILENAME", i, script.restorecon)

		for i in self.dirs.keys():
			newsh += re.sub("FILENAME", i, script.restorecon)

                for i in self.in_tcp + self.out_tcp:
                    if self.find_port(i) == None:
                        t1 = re.sub("PORTNUM", "%d" % i, script.tcp_ports)
                        newsh += re.sub("TEMPLATETYPE", self.name, t1)

                for i in self.in_udp + self.out_udp:
                    if self.find_port(i) == None:
			t1 = re.sub("PORTNUM", "%d" % i, script.udp_ports)
			newsh += re.sub("TEMPLATETYPE", self.name, t1)
			
		return newsh
	
	def write_te(self, out_dir):
		tefile = "%s/%s.te" % (out_dir, self.file_name)
		fd = open(tefile, "w")
		fd.write(self.generate_te())
		fd.close()
		return tefile

	def write_sh(self, out_dir):
		shfile = "%s/%s.sh" % (out_dir, self.file_name)
		fd = open(shfile, "w")
		fd.write(self.generate_sh())
		fd.close()
		return shfile

	def write_if(self, out_dir):
		iffile = "%s/%s.if" % (out_dir, self.file_name)
		fd = open(iffile, "w")
		fd.write(self.generate_if())
		fd.close()
		return iffile

	def write_fc(self,out_dir):
		fcfile = "%s/%s.fc" % (out_dir, self.file_name)
		fd = open(fcfile, "w")
		fd.write(self.generate_fc())
		fd.close()
		return fcfile

	def generate(self, out_dir = "."):
		out = "Created the following files:\n"
		out += "%-25s %s\n" % (_("Type Enforcment file"), self.write_te(out_dir))
		out += "%-25s %s\n" % (_("Interface file"), self.write_if(out_dir))
		out += "%-25s %s\n" % (_("File Contexts file"), self.write_fc(out_dir))
		out += "%-25s %s\n" % (_("Setup Script"),self.write_sh(out_dir))
		return out

def errorExit(error):
	sys.stderr.write("%s: " % sys.argv[0])
	sys.stderr.write("%s\n" % error)
	sys.stderr.flush()
	sys.exit(1)


if __name__ == '__main__':
    mypolicy = policy("cgi", "/var/www/cgi-bin/cgi", 3)
    mypolicy.set_in_tcp("513")
    mypolicy.set_in_udp("513")
    mypolicy.set_use_uid(True)
    mypolicy.set_use_tmp(True)
    mypolicy.set_use_syslog(True)
    mypolicy.set_use_pam(True)
    mypolicy.add_file("/var/lib/mysql/mysql.sock")
    mypolicy.add_file("/var/run/rpcbind.sock")
    mypolicy.add_file("/var/run/daemon.pub")
    mypolicy.add_file("/var/log/daemon.log")
    mypolicy.add_dir("/var/lib/daemon")
    mypolicy.add_dir("/etc/daemon")
    mypolicy.add_dir("/etc/daemon/special")
    mypolicy.set_out_tcp("8000")
    print mypolicy.generate()
#    mypolicy = policy("inetd", "/usr/sbin/inetd", 1)
#    mypolicy.generate()
#    mypolicy = policy("userapp", "/usr/sbin/userapp", 2)
#    mypolicy.generate()
#    mypolicy = policy("cgi", "cgi", 3)
#    mypolicy.generate()
    sys.exit(0)
    
	
