#!/usr/bin/python
#
# system-config-selinux.py - GUI for SELinux Config tool in system-config-selinux
#
# Dan Walsh <dwalsh@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import signal
import string
import gtk
import gtk.glade
import os
import libxml2
import gobject
import gnome
import sys
import polgen

##
## I18N
## 
PROGNAME="system-config-selinux"

import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME,
                    localedir="/usr/share/locale",
                    unicode=False,
                    codeset = 'utf-8')
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

gnome.program_init("SELinux Poligy Generation Tool", "5")

version = "1.0"

sys.path.append('/usr/share/system-config-selinux')
sys.path.append('.')

##
## Pull in the Glade file
##
if os.access("polgen.glade", os.F_OK):
    xml = gtk.glade.XML ("polgen.glade", domain=PROGNAME)
else:
    xml = gtk.glade.XML ("/usr/share/system-config-selinux/polgen.glade", domain=PROGNAME)

FILE = 1
DIR = 2
class childWindow:
    def __init__(self):
        self.xml = xml
        xml.signal_connect("on_druid_cancel", self.quit)
        xml.signal_connect("on_delete_clicked", self.delete)
        xml.signal_connect("on_exec_select_clicked", self.exec_select)
        xml.signal_connect("on_add_clicked", self.add)
        xml.signal_connect("on_add_dir_clicked", self.add_dir)
        xml.signal_connect("on_about_clicked", self.on_about_clicked)

    def verify(self, message, title="" ):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_YES_NO,
                                message)
        dlg.set_title(title)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc

    def info(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_OK,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        dlg.run()
        dlg.destroy()

    def error(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR,
                                gtk.BUTTONS_CLOSE,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        dlg.run()
        dlg.destroy()

    def get_type(self):
        if self.cgi_radiobutton.get_active():
            return polgen.policy.CGI
        if self.user_radiobutton.get_active():
            return polgen.policy.USER
        if self.init_radiobutton.get_active():
            return polgen.policy.Daemon
        if self.inetd_radiobutton.get_active():
            return polgen.policy.INETD
        
    def generate_policy(self, *args):
        try:
            my_policy=polgen.policy(self.name_entry.get_text(), self.exec_entry.get_text(), self.get_type())
            my_policy.set_in_tcp(self.in_tcp_entry.get_text())
            my_policy.set_in_udp(self.in_udp_entry.get_text())
            my_policy.set_out_tcp(self.out_tcp_entry.get_text())
            my_policy.set_out_udp(self.out_udp_entry.get_text())
            my_policy.set_use_syslog(self.syslog_checkbutton.get_active() == 1)
            my_policy.set_use_tmp(self.tmp_checkbutton.get_active() == 1)
            my_policy.set_use_uid(self.uid_checkbutton.get_active() == 1)
            my_policy.set_use_pam(self.pam_checkbutton.get_active() == 1)

            iter= self.store.get_iter_first()
            while(iter):
                if self.store.get_value(iter, 1) == FILE:
                    my_policy.add_file(self.store.get_value(iter, 0))
                else:
                    my_policy.add_dir(self.store.get_value(iter, 0))
                iter= self.store.iter_next(iter)
                
            self.info(my_policy.generate(self.output_entry.get_text()))
            return False
        except ValueError, e:
            self.error(e.message)
        
    def delete(self, args):
        store, iter = self.view.get_selection().get_selected()
        if iter != None:
            store.remove(iter)
            self.view.get_selection().select_path ((0,))

    def __add(self,type):
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        for i in self.file_dialog.get_filenames():
            iter = self.store.append()
            self.store.set_value(iter, 0, i)
            self.store.set_value(iter, 1, type)
        
    def exec_select(self, args):
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_title(_("Select executable file to be confined."))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.exec_entry.set_text(self.file_dialog.get_filename())

    def add(self, args):
        self.file_dialog.set_title(_("Select file(s) that confined application creates or writes"))
        self.file_dialog.set_select_multiple(1)
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.__add(FILE)

    def add_dir(self, args):
        self.file_dialog.set_title(_("Select directory(s) that the confined application owns and writes into"))
        self.file_dialog.set_select_multiple(0)
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER)
        self.__add(DIR)
        
    def on_about_clicked(self, args):
        dlg = xml.get_widget ("about_dialog")
        dlg.run ()
        dlg.hide ()

    def quit(self, args):
        gtk.main_quit()

    def setupScreen(self):
        # Bring in widgets from glade file.
        self.mainWindow = self.xml.get_widget("main_window")
        self.type = 0
        self.name_entry = self.xml.get_widget("name_entry")
        self.exec_entry = self.xml.get_widget("exec_entry")
        self.in_tcp_entry = self.xml.get_widget("in_tcp_entry")
        self.in_udp_entry = self.xml.get_widget("in_udp_entry")
        self.out_tcp_entry = self.xml.get_widget("out_tcp_entry")
        self.output_entry = self.xml.get_widget("output_entry")
        self.output_entry.set_text(os.getcwd())
        self.xml.get_widget("output_button").connect("clicked",self.output_button_clicked)
        
        self.out_udp_entry = self.xml.get_widget("out_udp_entry")
        self.user_radiobutton = self.xml.get_widget("user_radiobutton")
        self.init_radiobutton = self.xml.get_widget("init_radiobutton")
        self.inetd_radiobutton = self.xml.get_widget("inetd_radiobutton")
        self.cgi_radiobutton = self.xml.get_widget("cgi_radiobutton")
        self.tmp_checkbutton = self.xml.get_widget("tmp_checkbutton")
        self.uid_checkbutton = self.xml.get_widget("uid_checkbutton")
        self.pam_checkbutton = self.xml.get_widget("pam_checkbutton")
        self.syslog_checkbutton = self.xml.get_widget("syslog_checkbutton")
        self.view = self.xml.get_widget("write_treeview")
        self.file_dialog = self.xml.get_widget("filechooserdialog")
        self.xml.get_widget("name_page").connect("next",self.on_name_page_next)
        self.xml.get_widget("finish_page").connect("finish",self.generate_policy)

        self.store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.view.set_model(self.store)
        col = gtk.TreeViewColumn("",  gtk.CellRendererText(), text = 0)
        col.set_resizable(True)
        self.view.append_column(col)
        self.view.get_selection().select_path ((0,))

    def output_button_clicked(self, *args):
        self.file_dialog.set_title(_("Select directory to generate policy files in"))
        self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.file_dialog.set_select_multiple(0)
        rc = self.file_dialog.run()
        self.file_dialog.hide()
        if rc == gtk.RESPONSE_CANCEL:
            return
        self.output_entry.set_text(self.file_dialog.get_filename())
        
    def on_name_page_next(self, *args):
        name=self.name_entry.get_text()
        if name == "":
            self.error(_("You must enter a name"))
            return True
        
        exe = self.exec_entry.get_text()
        if exe == "":
            self.error(_("You must enter a executable"))
            return True
        
    def stand_alone(self):
        desktopName = _("Configue SELinux")

        self.setupScreen()
        self.mainWindow.connect("destroy", self.quit)

        self.mainWindow.show_all()
        gtk.main()

if __name__ == "__main__":
    signal.signal (signal.SIGINT, signal.SIG_DFL)

    app = childWindow()
    app.stand_alone()
