## usersPage.py - show selinux mappings
## Copyright (C) 2006 Red Hat, Inc.

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Dan Walsh
import string
import gtk
import gtk.glade
import os
import libxml2
import gobject
import sys
import commands
import seobject
from semanagePage import *;

##
## I18N
## 
PROGNAME="policycoreutils"
import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, localedir="/usr/share/locale", unicode=1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

class usersPage(semanagePage):
    def __init__(self, xml):
        semanagePage.__init__(self, xml, "users", "SELinux User")

        self.store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.view.set_model(self.store)
        self.store.set_sort_column_id(0, gtk.SORT_ASCENDING)        

        col = gtk.TreeViewColumn(_("SELinux\nUser"), gtk.CellRendererText(), text = 0)
        col.set_sort_column_id(0)
        col.set_resizable(True)
        self.view.append_column(col)

        col = gtk.TreeViewColumn(_("Labeling\nPrefix"), gtk.CellRendererText(), text = 1)
        col.set_resizable(True)
        self.view.append_column(col)
        col = gtk.TreeViewColumn(_("MLS/\nMCS Level"), gtk.CellRendererText(), text = 2)
        col.set_resizable(True)
        self.view.append_column(col)
        col = gtk.TreeViewColumn(_("MLS/\nMCS Range"), gtk.CellRendererText(), text = 3)
        col.set_resizable(True)
        self.view.append_column(col)

        col = gtk.TreeViewColumn(_("SELinux Roles"), gtk.CellRendererText(), text = 4)
        col.set_resizable(True)
        self.view.append_column(col)

        self.load()
        self.selinuxUserEntry = xml.get_widget("selinuxUserEntry")
        self.labelPrefixEntry = xml.get_widget("labelPrefixEntry")
        self.mlsLevelEntry = xml.get_widget("mlsLevelEntry")
        self.mlsRangeEntry = xml.get_widget("mlsRangeEntry")
        self.selinuxRolesEntry = xml.get_widget("selinuxRolesEntry")

    def load(self, filter = ""):
        self.filter=filter            
        self.user = seobject.seluserRecords()
        dict = self.user.get_all()
        keys = dict.keys()
        keys.sort()
        self.store.clear()
        for k in keys:
            level = seobject.translate(dict[k][1])
            range = seobject.translate(dict[k][2])
            if not (self.match(k, filter) or self.match(dict[k][0], filter) or self.match(level, filter) or self.match(range, filter) or self.match(dict[k][3], filter)):
                continue
            
            iter = self.store.append()
            self.store.set_value(iter, 0, k)
            self.store.set_value(iter, 1, dict[k][0])
            self.store.set_value(iter, 2, level)
            self.store.set_value(iter, 3, range)
            self.store.set_value(iter, 4, dict[k][3])
        self.view.get_selection().select_path ((0,))

    def delete(self):
        if semanagePage.delete(self) == gtk.RESPONSE_NO:
                return None

    def dialogInit(self):
        store, iter = self.view.get_selection().get_selected()
        self.selinuxUserEntry.set_text(store.get_value(iter, 0))
        self.selinuxUserEntry.set_sensitive(False)
        self.labelPrefixEntry.set_text(store.get_value(iter, 1))
        self.mlsLevelEntry.set_text(store.get_value(iter, 2))
        self.mlsRangeEntry.set_text(store.get_value(iter, 3))
        self.selinuxRolesEntry.set_text(store.get_value(iter, 4))
        protocol=store.get_value(iter, 2)
        
    def dialogClear(self):
        self.selinuxUserEntry.set_text("")
        self.selinuxUserEntry.set_sensitive(True)
        self.labelPrefixEntry.set_text("")
        self.mlsLevelEntry.set_text("s0")
        self.mlsRangeEntry.set_text("s0")
        self.selinuxRolesEntry.set_text("")
        
    def add(self):
        user = self.selinuxUserEntry.get_text()
        prefix = self.labelPrefixEntry.get_text()
        level = self.mlsLevelEntry.get_text()
        range = self.mlsRangeEntry.get_text()
        roles = self.selinuxRolesEntry.get_text()

        (rc, out) = commands.getstatusoutput("semanage user -a -R '%s' -r %s-%s -P %s %s" %  (roles, level, range, prefix, user))
        if rc != 0:
            self.error(out)
            return False
        iter = self.store.append()
        self.store.set_value(iter, 0, user)
        self.store.set_value(iter, 1, prefix)
        self.store.set_value(iter, 2, level)
        self.store.set_value(iter, 3, range)
        self.store.set_value(iter, 4, roles)
        
    def modify(self):
        user = self.selinuxUserEntry.get_text()
        prefix = self.labelPrefixEntry.get_text()
        level = self.mlsLevelEntry.get_text()
        range = self.mlsRangeEntry.get_text()
        roles = self.selinuxRolesEntry.get_text()

        (rc, out) = commands.getstatusoutput("semanage user -m -R '%s' -r %s-%s -P %s %s" %  (roles, level, range, prefix, user))

        if rc != 0:
            self.error(out)
            return False
        store, iter = self.view.get_selection().get_selected()
        iter = self.store.append()
        self.store.set_value(iter, 0, user)
        self.store.set_value(iter, 1, prefix)
        self.store.set_value(iter, 2, level)
        self.store.set_value(iter, 3, range)
        self.store.set_value(iter, 4, roles)

    def delete(self):
        store, iter = self.view.get_selection().get_selected()
        try:
            user=store.get_value(iter, 0)
            if user == "root" or user == "user_u":
                raise ValueError(_("SELinux user '%s' is required") % user)
                
            (rc, out) = commands.getstatusoutput("semanage user -d %s" %  user)
            if rc != 0:
                self.error(out)
                return False
            store.remove(iter)
            self.view.get_selection().select_path ((0,))
        except ValueError, e:
            self.error(e.args[0])

